<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="xml"/>

<xsl:template match="/emps">
   <!-- find the extreme values along both axes: mininum and maximum salary as well as minimum and maximum 'number of days'-->
   <xsl:variable name="the_max_sal">
     <xsl:for-each select="emp/sal">
       <xsl:sort data-type="number" order="descending"/>
       <xsl:if test="position()=1"><xsl:value-of select="."/></xsl:if>
     </xsl:for-each>
   </xsl:variable>
   <xsl:variable name="the_min_sal">
     <xsl:for-each select="emp/sal">
       <xsl:sort data-type="number" order="ascending"/>
       <xsl:if test="position()=1"><xsl:value-of select="."/></xsl:if>
     </xsl:for-each>
   </xsl:variable>
   <xsl:variable name="the_max_days">
     <xsl:for-each select="emp/days">
       <xsl:sort data-type="number" order="descending"/>
       <xsl:if test="position()=1"><xsl:value-of select="."/></xsl:if>
     </xsl:for-each>
   </xsl:variable>
   <xsl:variable name="the_min_days">
     <xsl:for-each select="emp/days">
       <xsl:sort data-type="number" order="ascending"/>
       <xsl:if test="position()=1"><xsl:value-of select="."/></xsl:if>
     </xsl:for-each>
   </xsl:variable>
    <xsl:element name="graphData">
      <xsl:element name="sets"> <!-- the graphData may contain more than one set; each set results in a different line in the graph -->
        <xsl:element name="set">    
          <xsl:attribute name="title">Initial Salary per hiredate</xsl:attribute>
          <xsl:attribute name="marker-type">star</xsl:attribute> <!-- several marker-types are supported, such as rectangle, square, circle, diamond and none -->
          <xsl:attribute name="color">red</xsl:attribute> <!-- color for the line and the markers for this set -->
          <xsl:for-each select="emp">
               <xsl:element name="measure">
                 <xsl:element name="xvalue"><xsl:value-of select="days"/></xsl:element>
                 <xsl:element name="yvalue"><xsl:value-of select="sal"/></xsl:element>
                 <!-- only add an xlabel if it is different from the previous xlabel -->
                 <xsl:if test="(preceding-sibling::emp[position()=1]/hirequarter != hirequarter) or (position()=1)">
                   <xsl:element name="xlabel"><xsl:value-of select="hirequarter"/></xsl:element> <!-- xlabel is the value displayed for this measure on the x-axis -->
                 </xsl:if>
                 <xsl:element name="label"><xsl:value-of select="concat(ename,' - ',hireday,' Salary: $',sal)"/></xsl:element> <!-- text displayed when the mouse moves over the measure's marker -->
                 <xsl:if test="ename = 'KING'" >
                   <xsl:element name="annotation">Expensive Guy, we had to pay a lot for him!</xsl:element> <!-- annotation to be displayed in the graph for this measure's marker -->
                   <xsl:element name="xgrid">true</xsl:element> <!-- show a vertical gridline -->
                 </xsl:if>
                 <xsl:if test="ename = 'SMITH'" >
                   <xsl:element name="annotation">He was the first and cheapest. I think he deserves a payrise!<xsl:value-of select="days"/></xsl:element>
                   <xsl:element name="xgrid">true</xsl:element>
                   <xsl:element name="ygrid">true</xsl:element> <!-- show a horizontal grid-line to the y-axis -->
                   <xsl:element name="ylabel"><xsl:value-of select="sal"/></xsl:element> <!-- label to display on the y-axis -->
                 </xsl:if>
               </xsl:element>
          </xsl:for-each>
      </xsl:element> <!-- set -->
    </xsl:element> <!-- sets -->
    <xsl:element name="xvalues"> <!-- set of values to display along the x-axis -->
      <xsl:element name="xvalue">
        <xsl:element name="value">-1461</xsl:element> <!-- value at which to place this x-axis marker -->
        <xsl:element name="label">1st January 1980</xsl:element> <!-- label to show -->
        <xsl:element name="gridline">true</xsl:element> <!-- show a vertical gridline -->
      </xsl:element>
      <xsl:element name="xvalue">
        <xsl:element name="value">-1</xsl:element>
        <xsl:element name="label">31st December 1983</xsl:element>
        <xsl:element name="gridline">true</xsl:element>
      </xsl:element>
    </xsl:element>
    <xsl:element name="yvalues">
      <xsl:element name="yvalue">
        <xsl:element name="value">1600</xsl:element>
        <xsl:element name="label">Union Lower Limit</xsl:element>
        <xsl:element name="gridline">true</xsl:element> <!-- show a horizontal gridline -->
      </xsl:element>
      <xsl:element name="ymarkers"> <!-- show markers along the y-axis starting at 0, moving in 5 steps up to 5000 -->
        <xsl:element name="minvalue">0</xsl:element>
        <xsl:element name="maxvalue">5000</xsl:element>
        <xsl:element name="steps">5</xsl:element>
        <xsl:element name="gridline">true</xsl:element>
      </xsl:element>
    </xsl:element> <!-- yvalues -->
    <xsl:element name="miny">0
      <!--  <xsl:value-of select="$the_min_sal"/> -->
    </xsl:element>
    <xsl:element name="maxy">
      <xsl:value-of select="$the_max_sal"/>
    </xsl:element>
    <xsl:element name="minx">-1461  <!-- 1st January 1980 -->
      <!-- <xsl:value-of select="$the_min_days"/> -->
    </xsl:element>
    <xsl:element name="maxx">-1 <!-- 31st December 1983 -->
    </xsl:element>
    <xsl:element name="title">Salaries of Employees vs. Hiredate</xsl:element>
    <xsl:element name="xtitle">Hiredates</xsl:element>
    <xsl:element name="ytitle">Salary in US$</xsl:element>
     </xsl:element> <!-- end of graphData -->
</xsl:template>

</xsl:stylesheet>